/*
 *	VDI functions
 *
 *	Johan Klockars
 */

#include <osbind.h>
#include <gemdefs.h>
#include <vdibind.h>
#include <aesbind.h>

#include <signal.h>

#include "doomdef.h"
#include "doomstat.h"
#include "v_video.h"
#include "m_argv.h"
#include "i_system.h"
#include "i_video.h"

#include "r_draw.h"
#include "am_map.h"
#include "f_wipe.h"
#include "f_finale.h"

#include "video/vdi.h"

extern byte     *orig_colormaps;

#define MAX_WIDTH 320
#define MAX_HEIGHT 200
static int      ap_id;
static int      vdi_handle;
static void     *buffer;
static MFDB     src_mfdb, dst_mfdb;
static int      blit_coords[8];
static unsigned char    vdi_index[256] = {0, 2, 3, 6, 4, 7, 5, 8, 9, 10, 11, 14, 12, 15, 13, 255, 0};

void I_VidInit_vdi(void)
{
        int scr_width, scr_height, width, height;
        int i,vdipixelsize;
        int work_in[12], work_out[57], dummy;
	int AllocFastRam=3; /* Alloc preferrably from Fast Ram */

        ap_id = appl_init();
        vdi_handle = graf_handle(&dummy, &dummy, &dummy, &dummy);
        for(i = 0; i < 10; i++)
                work_in[i] = 1;
        work_in[10] = 2;
        v_opnvwk(work_in, &vdi_handle, work_out);
        if ((width = scr_width = work_out[0] + 1) > MAX_WIDTH)
                width = MAX_WIDTH;
        if ((height = scr_height = work_out[1] + 1) > MAX_HEIGHT)
                height = MAX_HEIGHT;
        vq_extnd(vdi_handle, 1, work_out);
        bpp = work_out[4];

        /* --- Allocate screens */

	if ((width<SCREENWIDTH) || (height<SCREENHEIGHT))
		I_Error("You need a 320x200 screen or higher\n");
	if (bpp<8)
		I_Error("You need a 8,15,16 or 24 bits display\n");

	vdipixelsize=1;
	if ((bpp==15) || (bpp==16))
		vdipixelsize=2;
	if (bpp==24)
		vdipixelsize=3;
	if (bpp==32)
		vdipixelsize=4;

	/* Allocate buffer */

	if (M_CheckParm("-nofastblit"))
		AllocFastRam=0;	/* Alloc from normal Ram */

	buffer = Mxalloc(width * height * vdipixelsize, AllocFastRam);
	if (!buffer)
		I_Error("Not enough memory for draw buffer!\n");

        /* screens[1] = screens[0] = buffer; */

        memset(buffer, 0, width * height * vdipixelsize);
        src_mfdb.fd_w = width;
        src_mfdb.fd_h = height;
        src_mfdb.fd_wdwidth = width / 16;
        src_mfdb.fd_stand = 0;
        src_mfdb.fd_nplanes = bpp;
        dst_mfdb.fd_addr = 0;
        blit_coords[0] = 0;
        blit_coords[1] = 0;
        blit_coords[2] = width - 1;
        blit_coords[3] = height - 1;
        blit_coords[4] = (scr_width - width) / 2;
        blit_coords[5] = (scr_height - height) / 2;
        blit_coords[6] = blit_coords[4] + width - 1;
        blit_coords[7] = blit_coords[5] + height - 1;
}

void I_ShutdownGraphics_vdi(void)
{
        v_clsvwk(vdi_handle);
        appl_exit();

        Mfree(buffer);
}

void I_VidUpdate_vdi(void)
{
/*
        src_mfdb.fd_addr = screens[0];
*/
        vro_cpyfm(vdi_handle, 3, blit_coords, &src_mfdb, &dst_mfdb);
}

/*  */
/*  I_SetPalette */
/*  */
void I_SetPalette256_vdi(byte *palette)
{
	int i,rgb[3];

	if (!vdi_index[16]) {
		for(i = 16; i < 255; i++)
			vdi_index[i] = i;
		vdi_index[255] = 1;
	}

	for(i = 0; i < 256; i++)
	{
		int r, g, b;

		r = gammatable[usegamma][*palette++];   
		g = gammatable[usegamma][*palette++];
		b = gammatable[usegamma][*palette++];
                
		rgb[0] = (1000 * r) / 255;
		rgb[1] = (1000 * g) / 255;
		rgb[2] = (1000 * b) / 255;
		vs_color(vdi_handle, vdi_index[i], rgb);
	}
}

void I_InitGraphics_vdi(void)
{
	/* PM: Bug in NOVA VDI on Atari TT : it reports 16 bits for 32K mode */
	/* PM: reported by Guillaume Tello */
	/* PM: so use '-video-15' to force 15 bits routines if 16 bits reported */

	if ((bpp==16) && video15)
	{
		bpp=15;
	}

	switch (bpp)
	{
		case 8:
			R_DrawColumn=R_DrawColumn8;
			R_DrawColumnLow=R_DrawColumnLow8;
			R_DrawFuzzColumn=R_DrawFuzzColumn8;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow8;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn8;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow8;
			R_DrawSpan=R_DrawSpan8;
			R_DrawSpanLow=R_DrawSpanLow8;
			AM_DrawFline=AM_drawFline8;
			wipe_doMelt=wipe_doMelt8;
			V_DrawPatch=V_DrawPatch8;
			V_DrawPatchFlipped=V_DrawPatchFlipped8;
			F_DrawPatchCol=F_DrawPatchCol8;
			pixel_size=1;
			break;
		case 15:
			R_DrawColumn=R_DrawColumn16;
			R_DrawColumnLow=R_DrawColumnLow16;
			R_DrawFuzzColumn=R_DrawFuzzColumn16;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow16;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn16;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow16;
			R_DrawSpan=R_DrawSpan16;
			R_DrawSpanLow=R_DrawSpanLow16;
			AM_DrawFline=AM_drawFline16;
			wipe_doMelt=wipe_doMelt16;
			V_DrawPatch=V_DrawPatch16;
			V_DrawPatchFlipped=V_DrawPatchFlipped16;
			F_DrawPatchCol=F_DrawPatchCol16;
			pixel_size=2;
			fuzzmask=0x3DEF3DEF;
			break;
		case 16:
			R_DrawColumn=R_DrawColumn16;
			R_DrawColumnLow=R_DrawColumnLow16;
			R_DrawFuzzColumn=R_DrawFuzzColumn16;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow16;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn16;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow16;
			R_DrawSpan=R_DrawSpan16;
			R_DrawSpanLow=R_DrawSpanLow16;
			AM_DrawFline=AM_drawFline16;
			wipe_doMelt=wipe_doMelt16;
			V_DrawPatch=V_DrawPatch16;
			V_DrawPatchFlipped=V_DrawPatchFlipped16;
			F_DrawPatchCol=F_DrawPatchCol16;
			pixel_size=2;
			fuzzmask=0x7BEF7BEF;
			break;
		case 24:
			R_DrawColumn=R_DrawColumn24;
			R_DrawColumnLow=R_DrawColumnLow24;
			R_DrawFuzzColumn=R_DrawFuzzColumn24;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow24;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn24;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow24;
			R_DrawSpan=R_DrawSpan24;
			R_DrawSpanLow=R_DrawSpanLow24;
			AM_DrawFline=AM_drawFline24;
			wipe_doMelt=wipe_doMelt24;
			V_DrawPatch=V_DrawPatch24;
			V_DrawPatchFlipped=V_DrawPatchFlipped24;
			F_DrawPatchCol=F_DrawPatchCol24;
			pixel_size=3;
			break;
		case 32:
			I_Error("32 bit VDI operation not yet supported!\n");
			break;
	}

	signal(SIGINT, (void (*)(int)) I_Quit);
}

void I_SetMainScreen_vdi(void)
{
	src_mfdb.fd_addr = screens[0] = buffer;
}
