/* 	DOOM graphics stuff for Atari Falcon/TT Xbios. */
/* Patrice Mandin */

#include <signal.h>

#include "doomdef.h"
#include "doomstat.h"
#include "v_video.h"
#include "i_system.h"
#include "i_video.h"

#include "r_draw.h"
#include "am_map.h"
#include "f_wipe.h"
#include "f_finale.h"

#include "i_cookies.h"
#include "video/xbios.h"
/*

#include "m_argv.h"
#include "d_main.h"

#include "doomdef.h"
#include "r_local.h"

#include "z_zone.h"
*/

int VideoInited=0;

unsigned short	atari_oldvmode;
void		*atari_oldvbase;
unsigned long	atari_screen[2];	/* game screen */
void		*atari_ecralloc[2];

unsigned long	atari_offset;	/* offset for game screen in real screen */

unsigned short	oldTTpalette[256]; /* saved old TT palette */
unsigned long	oldF30palette[256];	/* saved old falcon palette */

void I_VidInit_xbios(void)
{
	int width,height;
	int i;

	/* --- Read video properties --- */

	width=SCREENWIDTH;
	height=SCREENHEIGHT;
	switch (cookie_vdo)
	{
		case VDO_TT:
			height=240;  	/* 480 8bits <-> 240 16 bits */
			atari_offset=SCREENWIDTH*40*2;
			break;
		case VDO_F30:
			switch (Montype())
			{
				case 0:		/* MONO */
					I_Error("Doom does not work on a monochrome monitor\n");
					break;
				case 1:		/* RVB : 320x200 */
				case 3:		/* TV : 320x200 */
					atari_offset=0;
					break;
				case 2:		/* VGA : 320x240 */
					atari_offset=SCREENWIDTH*20*2;
					height=240;
					break;
			}
			break;
	}

	/* --- Allocate screens */

	for (i=0;i<2;i++)
	{
		atari_ecralloc[i]=Mxalloc((width*height*2)+256,0);

		if (atari_ecralloc[i]==NULL)
			I_Error("Not enough memory for screens.\n");

		atari_screen[i]=(unsigned long )(atari_ecralloc[i]+16UL) & 0xFFFFFFF0UL;

		memset( (byte *)atari_screen[i],0,width*height*2);
	}
}

void I_ShutdownGraphics_xbios(void)
{
	/* Free screens */

	Mfree(atari_ecralloc[0]);
	Mfree(atari_ecralloc[1]);

	/* Switch back to old video mode */

	if (!VideoInited)
		return;

	switch(cookie_vdo)
	{
		case VDO_TT:
			Setscreen(-1,atari_oldvbase,-1,-1);
			EsetShift(atari_oldvmode);

			EsetPalette(0,256,oldTTpalette);
			break;
		case VDO_F30:
			Setscreen(-1,atari_oldvbase,-1,-1);
			Vsetmode(atari_oldvmode);

			VsetRGB(0,256,oldF30palette);
			break;
	}
}

void I_VidUpdate_xbios (void)
{
	/* Permutation des ecrans */

	if (pixel_size==1)
	{
		convert_c2p(screens[0],(byte *)(atari_screen[fbnum]+atari_offset),320,200,(cookie_vdo==VDO_TT));
	}

	Setscreen(-1,atari_screen[fbnum],-1,-1);
	Vsync();

	fbnum ^= 1;				
	if (dblbuffer)
	{
		screens[0]=(byte *)(atari_screen[fbnum]+atari_offset);
		R_InitBuffer(scaledviewwidth,viewheight); /* mettre a jour ylookup */
	}
}

/*  */
/*  I_SetPalette */
/*  */
void I_SetPalette256_xbios(byte *palette)
{
	int		i;
	int		r,v,b;
	unsigned long	tc=0;
	unsigned short	TT_palette[256];
	unsigned long	F30_palette[256];

	switch(cookie_vdo)
	{
		case VDO_TT:
			for (i=0;i<256;i++)
			{
				r = gammatable[usegamma][*palette++];	
				v = gammatable[usegamma][*palette++];	
				b = gammatable[usegamma][*palette++];	
					
				tc=((r>>4)<<8)|((v>>4)<<4)|(b>>4);
				TT_palette[i]=tc;
			}
			EsetPalette(0,256,TT_palette);
			break;
		case VDO_F30:
			for(i = 0; i < 256; i++)
			{
				r = gammatable[usegamma][*palette++];	
				v = gammatable[usegamma][*palette++];
				b = gammatable[usegamma][*palette++];

				F30_palette[i]=(r<<16)|(v<<8)|b;
			}
			VsetRGB(0,256,F30_palette);
			break;
	}
}

void I_InitGraphics_xbios(void)
{
	short	new_mode;
	int		i;
	unsigned long palettezero[256];

	for (i=0;i<256;i++)
		palettezero[i]=0;

	init_c2p();

	switch(cookie_vdo)
	{
		case VDO_TT:
			EgetPalette(0,256,oldTTpalette);

			atari_oldvbase=Logbase();
			atari_oldvmode=EgetShift();

			Setscreen(-1,atari_screen[1],-1,-1);
			EsetShift(TT_LOW);

			bpp=8;
			dblbuffer=false;

			EsetPalette(0,256,palettezero);
			break;
		case VDO_F30:
			VgetRGB(0,256,oldF30palette);

			atari_oldvmode=Vsetmode(-1);
			atari_oldvbase=Logbase();	

			new_mode=atari_oldvmode & ( VGA + PAL );
			if (video8)
			{
				new_mode|=BPS8;
				atari_offset>>=1; /* diviser par 2 */
				bpp=8;
				dblbuffer=false;
			} else {
				new_mode|=BPS16;
				bpp=16;
				dblbuffer=true;
			}
			if (Montype()==2)
				new_mode|=VERTFLAG;

			Setscreen(-1,atari_screen[1],-1,-1);
			Vsetmode(new_mode);

			VsetRGB(0,256,palettezero);
			break;
	}

	/* Init routines d'affichage */

	switch (bpp)
	{
		case 8:
			R_DrawColumn=R_DrawColumn8;
			R_DrawColumnLow=R_DrawColumnLow8;
			R_DrawFuzzColumn=R_DrawFuzzColumn8;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow8;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn8;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow8;
			R_DrawSpan=R_DrawSpan8;
			R_DrawSpanLow=R_DrawSpanLow8;
			AM_DrawFline=AM_drawFline8;
			wipe_doMelt=wipe_doMelt8;
			V_DrawPatch=V_DrawPatch8;
			V_DrawPatchFlipped=V_DrawPatchFlipped8;
			F_DrawPatchCol=F_DrawPatchCol8;
			pixel_size=1;
			break;
		case 15:
			R_DrawColumn=R_DrawColumn16;
			R_DrawColumnLow=R_DrawColumnLow16;
			R_DrawFuzzColumn=R_DrawFuzzColumn16;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow16;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn16;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow16;
			R_DrawSpan=R_DrawSpan16;
			R_DrawSpanLow=R_DrawSpanLow16;
			AM_DrawFline=AM_drawFline16;
			wipe_doMelt=wipe_doMelt16;
			V_DrawPatch=V_DrawPatch16;
			V_DrawPatchFlipped=V_DrawPatchFlipped16;
			F_DrawPatchCol=F_DrawPatchCol16;
			pixel_size=2;
			fuzzmask=0x3DEF3DEF;
			break;
		case 16:
			R_DrawColumn=R_DrawColumn16;
			R_DrawColumnLow=R_DrawColumnLow16;
			R_DrawFuzzColumn=R_DrawFuzzColumn16;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow16;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn16;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow16;
			R_DrawSpan=R_DrawSpan16;
			R_DrawSpanLow=R_DrawSpanLow16;
			AM_DrawFline=AM_drawFline16;
			wipe_doMelt=wipe_doMelt16;
			V_DrawPatch=V_DrawPatch16;
			V_DrawPatchFlipped=V_DrawPatchFlipped16;
			F_DrawPatchCol=F_DrawPatchCol16;
			pixel_size=2;
			fuzzmask=0x7BEF7BEF;
			break;
		case 24:
			R_DrawColumn=R_DrawColumn24;
			R_DrawColumnLow=R_DrawColumnLow24;
			R_DrawFuzzColumn=R_DrawFuzzColumn24;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow24;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn24;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow24;
			R_DrawSpan=R_DrawSpan24;
			R_DrawSpanLow=R_DrawSpanLow24;
			AM_DrawFline=AM_drawFline24;
			wipe_doMelt=wipe_doMelt24;
			V_DrawPatch=V_DrawPatch24;
			V_DrawPatchFlipped=V_DrawPatchFlipped24;
			F_DrawPatchCol=F_DrawPatchCol24;
			pixel_size=3;
			break;
	}

	VideoInited=1;

	signal(SIGINT, (void (*)(int)) I_Quit);
}

void I_SetMainScreen_xbios(void)
{
	/* Put screens[0] on our screen */

	if (pixel_size!=1)
		screens[0]=(byte *)(atari_screen[1]+atari_offset);
}
